@@ -38,7 +38,6 @@
 
 typedef enum {
     NUMBER       = (1 << 3),   /* Number button */
-    FUNC         = (1 << 6),   /* Function */
     PREFIXOP     = (1 << 15),  /* Unary prefix operation */
 } ButtonFlags;
 
@@ -52,6 +51,7 @@ typedef struct {
 /* Note that none of these strings can be translated as the parser expects them to be correct */
 /* id, symname flags */
 static Function functions[NFUNCTIONS] = {
+{ FN_SPACE,             " ", 0},
 { FN_0,                 "0", NUMBER },
 { FN_1,                 "1", NUMBER },
 { FN_2,                 "2", NUMBER },    
@@ -81,56 +81,57 @@ static Function functions[NFUNCTIONS] = {
 { FN_BACKSPACE,         NULL, 0 },
 { FN_DELETE,            NULL, 0 },
 { FN_CHANGE_SIGN,       NULL, 0 },
-{ FN_INTEGER,           "Int", FUNC },
-{ FN_FRACTION,          "Frac", FUNC },
+{ FN_INTEGER,           " int ", 0 },
+{ FN_FRACTION,          " frac ", 0 },
 { FN_PERCENTAGE,        "%", 0 },
 { FN_SQUARE,            "²", 0 },
 { FN_SQUARE_ROOT,       "√", 0 },
 { FN_RECIPROCAL,        NULL, 0 },
-{ FN_E_POW_X,           "e^", PREFIXOP },
-{ FN_10_POW_X,          "10^", PREFIXOP },       
-{ FN_2_POW_X,           "2^", PREFIXOP },
+{ FN_E_POW_X,           "e^", 0 },
+{ FN_10_POW_X,          "10^", 0 },       
+{ FN_2_POW_X,           "2^", 0 },
 { FN_X_POW_Y,           "^", 0 },
 { FN_X_POW_Y_INV,       "^(1/(", 0 },
 { FN_FACTORIAL,         "!", 0 },
-{ FN_RANDOM,            "Rand", 0 },
-{ FN_SIN,               "Sin", FUNC },
-{ FN_SINH,              "Sinh", FUNC },
-{ FN_ASIN,              "Asin", FUNC },
-{ FN_ASINH,             "Asinh", FUNC },
-{ FN_COS,               "Cos", FUNC },
-{ FN_COSH,              "Cosh", FUNC },
-{ FN_ACOS,              "Acos", FUNC },
-{ FN_ACOSH,             "Acosh", FUNC },
-{ FN_TAN,               "Tan", FUNC },
-{ FN_TANH,              "Tanh", FUNC },
-{ FN_ATAN,              "Atan", FUNC },
-{ FN_TAN,               "Atanh", FUNC },
-{ FN_NATURAL_LOGARITHM, "Ln", FUNC },
-{ FN_LOGARITHM,         "Log", FUNC },
-{ FN_LOGARITHM2,        "Log2", FUNC },
-{ FN_ABSOLUTE_VALUE,    "Abs", FUNC },
-{ FN_TRUNC,             "Trunc", FUNC },
+{ FN_RANDOM,            " rand ", 0 },
+{ FN_SIN,               " sin ", 0 },
+{ FN_SINH,              " sinh ", 0 },
+{ FN_ASIN,              " sin⁻¹ ", 0 },
+{ FN_ASINH,             " asinh ", 0 },
+{ FN_COS,               " cos ", 0 },
+{ FN_COSH,              " cosh ", 0 },
+{ FN_ACOS,              " cos⁻¹ ", 0 },
+{ FN_ACOSH,             " acosh ", 0 },
+{ FN_TAN,               " tan ", 0 },
+{ FN_TANH,              " tanh ", 0 },
+{ FN_ATAN,              " tan⁻¹ ", 0 },
+{ FN_TAN,               " atanh ", 0 },
+{ FN_NATURAL_LOGARITHM, " ln ", 0 },
+{ FN_LOGARITHM,         " log ", 0 },
+{ FN_LOGARITHM2,        " log₂ ", 0 },
+{ FN_ABSOLUTE_VALUE,    "|", 0 },
+{ FN_ABSOLUTE_VALUE_FUNC, " abs ", 0 },    
+{ FN_TRUNC,             " trunc ", 0 },
 { FN_MODULUS_DIVIDE,    " mod ", 0 },
-{ FN_1S_COMPLEMENT,     "1s", FUNC },
-{ FN_2S_COMPLEMENT,     "2s", FUNC },
+{ FN_1S_COMPLEMENT,     " ones ", 0 },
+{ FN_2S_COMPLEMENT,     " twos ", 0 },
 { FN_EXPONENTIAL,       "e", 0 },
 { FN_NOT,               "~", 0 },
-{ FN_OR,                " OR ", 0 },
-{ FN_AND,               " AND ", 0 },       
-{ FN_XOR,               " XOR ", 0 },
-{ FN_XNOR,              " XNOR ", 0 },
+{ FN_OR,                " or ", 0 },
+{ FN_AND,               " and ", 0 },       
+{ FN_XOR,               " xor ", 0 },
+{ FN_XNOR,              " xnor ", 0 },
 { FN_TOGGLE_BIT,        NULL, 0 },
-{ FN_FINC_CTRM,         "Ctrm", 0 },
-{ FN_FINC_DDB,          "Ddb", 0 },
-{ FN_FINC_FV,           "Fv", 0 },
-{ FN_FINC_GPM,          "Gpm", 0 },
-{ FN_FINC_PMT,          "Pmt", 0 },
-{ FN_FINC_PV,           "Pv", 0 },
-{ FN_FINC_RATE,         "Rate", 0 },
-{ FN_FINC_SLN,          "Sln", 0 },
-{ FN_FINC_SYD ,         "Syd", 0 },
-{ FN_FINC_TERM,         "Term", 0 },
+{ FN_FINC_CTRM,         NULL, 0 },
+{ FN_FINC_DDB,          NULL, 0 },
+{ FN_FINC_FV,           NULL, 0 },
+{ FN_FINC_GPM,          NULL, 0 },
+{ FN_FINC_PMT,          NULL, 0 },
+{ FN_FINC_PV,           NULL, 0 },
+{ FN_FINC_RATE,         NULL, 0 },
+{ FN_FINC_SLN,          NULL, 0 },
+{ FN_FINC_SYD ,         NULL, 0 },
+{ FN_FINC_TERM,         NULL, 0 },
 { FN_SHIFT,             NULL, 0 },
 { FN_STORE,             NULL, 0 },
 { FN_RECALL,            NULL, 0 },
@@ -168,6 +169,53 @@ do_accuracy(int value)
 }
 
 
+static int
+get_variable(const char *name, MPNumber *z, void *data)
+{
+    char *c, *lower_name;
+    int result = 1;
+    
+    lower_name = strdup(name);
+    for (c = lower_name; *c; c++)
+        *c = tolower(*c);
+
+    if (lower_name[0] == 'r')
+        register_get(atoi(name+1), z);
+    else if (strcmp(lower_name, "ans") == 0)
+        mp_set_from_mp(display_get_answer(&v->display), z);
+    else
+        result = 0;
+
+    free(lower_name);
+
+    return result;
+}
+
+
+static void
+set_variable(const char *name, const MPNumber *x, void *data)
+{
+    if (name[0] == 'R' || name[0] == 'r')
+        register_set(atoi(name+1), x);
+}
+
+
+static int
+parse(const char *text, MPNumber *z)
+{
+    MPEquationOptions options;
+
+    memset(&options, 0, sizeof(options));
+    options.base = v->base;
+    options.wordlen = v->wordlen;
+    options.angle_units = v->ttype;
+    options.get_variable = get_variable;
+    options.set_variable = set_variable;
+
+    return mp_equation_parse(text, &options, z);
+}
+
+
 /* Perform a user defined function. */
 static void
 do_function(int index)
@@ -177,8 +225,9 @@ do_function(int index)
     assert(index >= 0);
     assert(index <= 9);
 
-    ret = mp_equation_udf_parse(function_get_value(index));
-    if (!ret) {
+    ret = parse(function_get_value(index), display_get_answer(&v->display));
+    if (ret == 0) {
+        display_set_answer(&v->display);
         ui_set_statusbar("", "");
     } else {
         /* Translators: This message is displayed in the status bar when an
@@ -265,16 +314,16 @@ do_insert_character(const char *text)
 static void
 do_shift(int count)
 {
-    MPNumber MPval;
+    MPNumber z;
 
-    if (display_is_usable_number(&v->display, &MPval) || !mp_is_integer(&MPval)) {
+    if (display_is_usable_number(&v->display, &z) || !mp_is_integer(&z)) {
         /* Translators: This message is displayed in the status bar when a bit
            shift operation is performed and the display does not contain a number */
         ui_set_statusbar(_("No sane value to do bitwise shift"),
                          "gtk-dialog-error");
     }
     else {
-        mp_shift(&MPval, count, display_get_answer(&v->display));
+        mp_shift(&z, count, display_get_answer(&v->display));
         display_set_answer(&v->display);
     }
 }
@@ -282,26 +331,26 @@ do_shift(int count)
 
 /* Change the current base setting. */
 static void
-do_base(BaseType b)
+do_base(int b)
 {
     int ret;
-    MPNumber MP;
+    MPNumber z;
 
     if (!display_is_empty(&v->display))
     {   
-        ret = display_is_usable_number(&v->display, &MP);
+        ret = display_is_usable_number(&v->display, &z);
         if (ret) {
             ui_set_statusbar(_("No sane value to convert"),
                              "gtk-dialog-error");
         } else {
-            mp_set_from_mp(&MP, display_get_answer(&v->display));
+            mp_set_from_mp(&z, display_get_answer(&v->display));
             display_set_answer(&v->display);
             clear_undo_history();
         }
     }
     v->base = b;
-    set_enumerated_resource(R_BASE, Rbstr, (int) v->base);
-    display_set_base(&v->display, basevals[v->base]);
+    set_int_resource(R_BASE, v->base);
+    display_set_base(&v->display, v->base);
     ui_set_base(v->base);
     ui_make_registers();
 }
@@ -311,15 +360,15 @@ do_base(BaseType b)
 static void
 do_exchange(int index)
 {
-    MPNumber MPtemp, MPexpr;
+    MPNumber r, z;
 
-    if (display_is_usable_number(&v->display, &MPexpr)) {
+    if (display_is_usable_number(&v->display, &z)) {
         ui_set_statusbar(_("No sane value to store"),
                          "gtk-dialog-error");
     } else {
-        register_get(index, &MPtemp);
-        register_set(index, &MPexpr);
-        mp_set_from_mp(&MPtemp, display_get_answer(&v->display));
+        register_get(index, &r);
+        register_set(index, &z);
+        mp_set_from_mp(&r, display_get_answer(&v->display));
         display_set_answer(&v->display);
         ui_make_registers();
     }
@@ -339,17 +388,17 @@ static void
 do_numtype(DisplayFormat n)   /* Set number display type. */
 {
     int ret;
-    MPNumber MP;
+    MPNumber z;
 
     /* Convert display if it contains a number */
     if (!display_is_empty(&v->display))
     {
-        ret = display_is_usable_number(&v->display, &MP);
+        ret = display_is_usable_number(&v->display, &z);
         if (ret) {
             ui_set_statusbar(_("No sane value to convert"),
                              "gtk-dialog-error");
         } else {
-            mp_set_from_mp(&MP, display_get_answer(&v->display));
+            mp_set_from_mp(&z, display_get_answer(&v->display));
             display_set_answer(&v->display);
             clear_undo_history();
         }
@@ -420,7 +469,7 @@ do_expression(int function, int arg, int cursor_start, int cursor_end)
 
         case FN_SHIFT:
             do_shift(arg);
-            return;
+            break;
 
         case FN_SET_ACCURACY:
             do_accuracy(arg);
@@ -445,7 +494,7 @@ do_expression(int function, int arg, int cursor_start, int cursor_end)
         
         case FN_FUNCTION:
             do_function(arg);
-            return;
+            break;
         
         case FN_PASTE:
             do_paste(cursor_start, cursor_end, (const char *)arg); // FIXME: Probably not 64 bit safe
@@ -519,14 +568,14 @@ do_expression(int function, int arg, int cursor_start, int cursor_end)
                 
             /* Solve the equation */
             } else {
-                MPNumber MPval;
+                MPNumber z;
                 int result;
                 const char *message = NULL;
-                
-                result = display_solve(&v->display, &MPval);
+
+                result = parse(display_get_text(&v->display), &z);
                 switch (result) {
                     case 0:
-                        mp_set_from_mp(&MPval, ans);
+                        mp_set_from_mp(&z, ans);
                         display_set_answer(&v->display);
                         break;
 
@@ -559,8 +608,20 @@ do_expression(int function, int arg, int cursor_start, int cursor_end)
                        message = _("Overflow. Try a bigger word size");
                        break;
 
-                    case -MPMATH_ERR:
-                        message = v->math_error_text;
+                    case -PARSER_ERR_UNKNOWN_VARIABLE:
+                        /* Translators; Error displayd to user when they
+                         * an unknown variable is entered */
+                       message = _("Unknown variable");
+                       break;
+
+                    case -PARSER_ERR_UNKNOWN_FUNCTION:
+                        /* Translators; Error displayd to user when they
+                         * an unknown function is entered */
+                       message = _("Unknown function");
+                       break;
+
+                    case -PARSER_ERR_MP:
+                        message = mp_get_error();
                         break;
 
                     default:
@@ -575,18 +636,7 @@ do_expression(int function, int arg, int cursor_start, int cursor_end)
             break;
 
         default:
-            /* If display is a number then perform functions on that number */
-            if (functions[function].flags & (PREFIXOP | FUNC) && display_is_result(&v->display)) {
-                SNPRINTF(buf, MAXLINE, "%s(", functions[function].symname);
-                display_surround(&v->display, buf, ")");
-            } else {
-                if (functions[function].flags & FUNC) {
-                    SNPRINTF(buf, MAXLINE, "%s(", functions[function].symname);
-                    display_insert(&v->display, cursor_start, cursor_end, buf);
-                } else {
-                    display_insert(&v->display, cursor_start, cursor_end, functions[function].symname);
-                }
-            }
+            display_insert(&v->display, cursor_start, cursor_end, functions[function].symname);
             break;
     }
 
